@@ -19,19 +19,24 @@
 #ifndef KDESKTOPFILETEST_H
 #define KDESKTOPFILETEST_H
 
-#include <QtCore/QObject>
+#include <QObject>
 
 class KDesktopFileTest : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
+    void initTestCase();
     void testRead();
+    void testReadDirectory();
     void testReadLocalized_data();
     void testReadLocalized();
     void testUnsuccessfulTryExec();
     void testSuccessfulTryExec();
     void testActionGroup();
     void testIsAuthorizedDesktopFile();
+    void testTryExecWithAuthorizeAction();
+    void testLocateLocal_data();
+    void testLocateLocal();
 
 };
 
