@@ -26,14 +26,14 @@
 // Compiling this file with this flag is just crazy
 #undef QT_NO_CAST_FROM_ASCII
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QSettings>
-#include <QtCore/QTextStream>
-#include <QtXml/QDomAttr>
-#include <QtCore/QRegExp>
-#include <QtCore/QStringList>
+#include <QCoreApplication>
+#include <QFile>
+#include <QFileInfo>
+#include <QSettings>
+#include <QTextStream>
+#include <QDomAttr>
+#include <QRegExp>
+#include <QStringList>
 
 #include <qcommandlineparser.h>
 #include <qcommandlineoption.h>
@@ -776,11 +776,11 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
             paramType = e.attribute(QStringLiteral("type"));
             if (param.isEmpty()) {
                 cerr << "Parameter must have a name: " << dumpNode(e) << endl;
-                return 0;
+                return nullptr;
             }
             if (paramType.isEmpty()) {
                 cerr << "Parameter must have a type: " << dumpNode(e) << endl;
-                return 0;
+                return nullptr;
             }
             if ((paramType == QLatin1String("Int")) || (paramType == QLatin1String("UInt"))) {
                 bool ok;
@@ -788,7 +788,7 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
                 if (!ok) {
                     cerr << "Integer parameter must have a maximum (e.g. max=\"0\"): "
                          << dumpNode(e) << endl;
-                    return 0;
+                    return nullptr;
                 }
             } else if (paramType == QLatin1String("Enum")) {
                 for (QDomElement e2 = e.firstChildElement(); !e2.isNull(); e2 = e2.nextSiblingElement()) {
@@ -804,13 +804,13 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
                 if (paramValues.isEmpty()) {
                     cerr << "No values specified for parameter '" << param
                          << "'." << endl;
-                    return 0;
+                    return nullptr;
                 }
                 paramMax = paramValues.count() - 1;
             } else {
                 cerr << "Parameter '" << param << "' has type " << paramType
                      << " but must be of type int, uint or Enum." << endl;
-                return 0;
+                return nullptr;
             }
         } else if (tag == QLatin1String("default")) {
             if (e.attribute(QStringLiteral("param")).isEmpty()) {
@@ -849,7 +849,6 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
             }
             choices = CfgEntry::Choices(chlist, name, prefix);
         } else if (tag == QLatin1String("emit")) {
-            QDomNode signalNode;
             Signal signal;
             signal.name = e.attribute(QStringLiteral("signal"));
             signalList.append(signal);
@@ -866,7 +865,7 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
     bool nameIsEmpty = name.isEmpty();
     if (nameIsEmpty && key.isEmpty()) {
         cerr << "Entry must have a name or a key: " << dumpNode(element) << endl;
-        return 0;
+        return nullptr;
     }
 
     if (key.isEmpty()) {
@@ -884,12 +883,12 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
     if (name.contains(QStringLiteral("$("))) {
         if (param.isEmpty()) {
             cerr << "Name may not be parameterized: " << name << endl;
-            return 0;
+            return nullptr;
         }
     } else {
         if (!param.isEmpty()) {
             cerr << "Name must contain '$(" << param << ")': " << name << endl;
-            return 0;
+            return nullptr;
         }
     }
 
@@ -924,13 +923,13 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
                     i = paramValues.indexOf(index);
                     if (i == -1) {
                         cerr << "Index '" << index << "' for default value is unknown." << endl;
-                        return 0;
+                        return nullptr;
                     }
                 }
 
                 if ((i < 0) || (i > paramMax)) {
                     cerr << "Index '" << i << "' for default value is out of range [0, " << paramMax << "]." << endl;
-                    return 0;
+                    return nullptr;
                 }
 
                 QString tmpDefaultValue = e.text();
@@ -951,7 +950,7 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
         else {
             cerr << "The name '" << name << "' is not a valid name for an entry." << endl;
         }
-        return 0;
+        return nullptr;
     }
 
     if (allNames.contains(name)) {
@@ -961,7 +960,7 @@ CfgEntry *parseEntry(const QString &group, const QDomElement &element, const Cfg
         else {
             cerr << "The name '" << name << "' is not unique." << endl;
         }
-        return 0;
+        return nullptr;
     }
     allNames.append(name);
 
@@ -1893,6 +1892,7 @@ int main(int argc, char **argv)
         h << "    static " << cfg.className << " *self();" << endl;
         if (cfgFileNameArg) {
             h << "    static void instance(const QString& cfgfilename);" << endl;
+            h << "    static void instance(KSharedConfig::Ptr config);" << endl;
         }
     }
 
@@ -2115,14 +2115,14 @@ int main(int argc, char **argv)
     if (cfg.singleton) {
         h << "    " << cfg.className << "(";
         if (cfgFileNameArg) {
-            h << "const QString& arg";
+            h << "KSharedConfig::Ptr config";
         }
         h << ");" << endl;
         h << "    friend class " << cfg.className << "Helper;" << endl << endl;
     }
 
     if (hasNonModifySignals) {
-        h << "    bool usrSave() Q_DECL_OVERRIDE;" << endl;
+        h << "    bool usrSave() override;" << endl;
     }
 
     // Member variables
@@ -2243,7 +2243,7 @@ int main(int argc, char **argv)
 
     // Header required by singleton implementation
     if (cfg.singleton) {
-        cpp << "#include <qglobal.h>" << endl << "#include <QtCore/QFile>" << endl << endl;
+        cpp << "#include <qglobal.h>" << endl << "#include <QFile>" << endl << endl;
     }
     if (cfg.singleton && cfgFileNameArg) {
         cpp << "#include <QDebug>" << endl << endl;
@@ -2298,7 +2298,7 @@ int main(int argc, char **argv)
         cpp << "class " << cfg.className << "Helper" << endl;
         cpp << '{' << endl;
         cpp << "  public:" << endl;
-        cpp << "    " << cfg.className << "Helper() : q(0) {}" << endl;
+        cpp << "    " << cfg.className << "Helper() : q(nullptr) {}" << endl;
         cpp << "    ~" << cfg.className << "Helper() { delete q; }" << endl;
         cpp << "    " << cfg.className << " *q;" << endl;
         cpp << "};" << endl;
@@ -2320,15 +2320,25 @@ int main(int argc, char **argv)
         cpp << "}" << endl << endl;
 
         if (cfgFileNameArg) {
-            cpp << "void " << cfg.className << "::instance(const QString& cfgfilename)" << endl;
-            cpp << "{" << endl;
-            cpp << "  if (s_global" << cfg.className << "()->q) {" << endl;
-            cpp << "     qDebug() << \"" << cfg.className << "::instance called after the first use - ignoring\";" << endl;
-            cpp << "     return;" << endl;
-            cpp << "  }" << endl;
-            cpp << "  new " << cfg.className << "(cfgfilename);" << endl;
-            cpp << "  s_global" << cfg.className << "()->q->read();" << endl;
-            cpp << "}" << endl << endl;
+            auto instance = [&cfg, &cpp] (const QString &type, const QString arg, bool wrap) {
+                cpp << "void " << cfg.className << "::instance(" << type << " " << arg << ")" << endl;
+                cpp << "{" << endl;
+                cpp << "  if (s_global" << cfg.className << "()->q) {" << endl;
+                cpp << "     qDebug() << \"" << cfg.className << "::instance called after the first use - ignoring\";" << endl;
+                cpp << "     return;" << endl;
+                cpp << "  }" << endl;
+                cpp << "  new " << cfg.className << "(";
+                if (wrap) {
+                    cpp << "KSharedConfig::openConfig(" << arg << ")";
+                } else {
+                    cpp << arg;
+                }
+                cpp << ");" << endl;
+                cpp << "  s_global" << cfg.className << "()->q->read();" << endl;
+                cpp << "}" << endl << endl;
+            };
+            instance(QStringLiteral("const QString&"), QStringLiteral("cfgfilename"), true);
+            instance(QStringLiteral("KSharedConfig::Ptr"), QStringLiteral("config"), false);
         }
     }
 
@@ -2339,7 +2349,7 @@ int main(int argc, char **argv)
     // Constructor
     cpp << cfg.className << "::" << cfg.className << "( ";
     if (cfgFileNameArg) {
-        if (!cfg.singleton && ! cfg.forceStringFilename) {
+        if (! cfg.forceStringFilename) {
             cpp << " KSharedConfig::Ptr config";
         } else {
             cpp << " const QString& config";
@@ -2599,7 +2609,7 @@ int main(int argc, char **argv)
         cpp << "  delete d;" << endl;
     }
     if (cfg.singleton) {
-        cpp << "  s_global" << cfg.className << "()->q = 0;" << endl;
+        cpp << "  s_global" << cfg.className << "()->q = nullptr;" << endl;
     }
     cpp << "}" << endl << endl;
 
